"""add images

Revision ID: aaa1b50ab63e
Revises: 39659baca9b3
Create Date: 2024-01-29 20:21:43.012903

"""
from alembic import op
import sqlalchemy as sa


# revision identifiers, used by Alembic.
revision = 'aaa1b50ab63e'
down_revision = '39659baca9b3'
branch_labels = None
depends_on = None


def upgrade() -> None:
    # ### commands auto generated by Alembic - please adjust! ###
    op.add_column('recipe', sa.Column('thumbnail_image', sa.String(length=100), nullable=True))
    op.add_column('recipe', sa.Column('detail_image', sa.String(length=100), nullable=True))
    # ### end Alembic commands ###


def downgrade() -> None:
    # ### commands auto generated by Alembic - please adjust! ###
    op.drop_column('recipe', 'detail_image')
    op.drop_column('recipe', 'thumbnail_image')
    # ### end Alembic commands ###
