if(!self.define){let e,i={};const n=(n,s)=>(n=new URL(n+".js",s).href,i[n]||new Promise((i=>{if("document"in self){const e=document.createElement("script");e.src=n,e.onload=i,document.head.appendChild(e)}else e=n,importScripts(n),i()})).then((()=>{let e=i[n];if(!e)throw new Error(`Module ${n} didn’t register its module`);return e})));self.define=(s,r)=>{const c=e||("document"in self?document.currentScript.src:"")||location.href;if(i[c])return;let t={};const o=e=>n(e,c),d={module:{uri:c},exports:t,require:o};i[c]=Promise.all(s.map((e=>d[e]||o(e)))).then((e=>(r(...e),t)))}}define(["./workbox-3e911b1d"],(function(e){"use strict";self.skipWaiting(),e.clientsClaim(),e.precacheAndRoute([{url:"assets/index-427af6d5.js",revision:null},{url:"assets/index-f21a461b.css",revision:null},{url:"index.html",revision:"1bc6c1a032dacc6d550c87ade495ec85"},{url:"registerSW.js",revision:"1872c500de691dce40960bb85481de07"},{url:"maskable-icon-512x512.png",revision:"dcad8bd2a7b9323d3c83c115ad979ce6"},{url:"pwa-192x192.png",revision:"4acac52f485739322eff1fcc49ff0126"},{url:"pwa-512x512.png",revision:"03fee140dbf59b66e5841a99e9c4b5fa"},{url:"manifest.webmanifest",revision:"0b847c2ed344e2953c7572b27ad1b80a"}],{}),e.cleanupOutdatedCaches(),e.registerRoute(new e.NavigationRoute(e.createHandlerBoundToURL("index.html")))}));
