"""Rename fibres.

Revision ID: b8b797b55d1b
Revises: f91610b78d55
Create Date: 2021-04-10 13:32:03.840335

"""
from alembic import op
import sqlalchemy as sa


# revision identifiers, used by Alembic.
revision = 'b8b797b55d1b'
down_revision = 'f91610b78d55'
branch_labels = None
depends_on = None


def upgrade():
    # ### commands auto generated by Alembic - please adjust! ###
    op.add_column('ingredient', sa.Column('fibres', sa.Float(), nullable=True))
    with op.batch_alter_table('ingredient') as batch_op:
        batch_op.drop_column('fibers')
    # ### end Alembic commands ###


def downgrade():
    # ### commands auto generated by Alembic - please adjust! ###
    op.add_column('ingredient', sa.Column('fibers', sa.Float(), nullable=True))
    with op.batch_alter_table('ingredient') as batch_op:
        batch_op.drop_column('fibres')
    # ### end Alembic commands ###
