"""tweak ingredient item

Revision ID: c7b802139fbf
Revises: aaa1b50ab63e
Create Date: 2024-06-24 15:03:02.852874

"""
from alembic import op
import sqlalchemy as sa


# revision identifiers, used by Alembic.
revision = 'c7b802139fbf'
down_revision = 'aaa1b50ab63e'
branch_labels = None
depends_on = None


def upgrade() -> None:
    # ### commands auto generated by Alembic - please adjust! ###
    pass
    # ### end Alembic commands ###


def downgrade() -> None:
    # ### commands auto generated by Alembic - please adjust! ###
    pass
    # ### end Alembic commands ###
