"""add ingredient item order

Revision ID: ea9e0ff5a7ff
Revises: c7b802139fbf
Create Date: 2024-09-23 20:07:17.783503

"""
from alembic import op
import sqlalchemy as sa


# revision identifiers, used by Alembic.
revision = 'ea9e0ff5a7ff'
down_revision = 'c7b802139fbf'
branch_labels = None
depends_on = None


def upgrade() -> None:
    # ### commands auto generated by Alembic - please adjust! ###
    op.add_column('ingredient_item', sa.Column('order', sa.Integer(), nullable=True))
    # ### end Alembic commands ###


def downgrade() -> None:
    # ### commands auto generated by Alembic - please adjust! ###
    op.drop_column('ingredient_item', 'order')
    # ### end Alembic commands ###
